@@ -978,6 +978,15 @@ int raft_fixture_init(struct raft_fixture *f, unsigned n, struct raft_fsm *fsms)
     return 0;
 }
 
+int raft_fixture_initialize(struct raft_fixture *f)
+{
+    f->time = 0;
+    logInit(&f->log);
+    f->commit_index = 0;
+    f->hook = NULL;
+    return 0;
+}
+
 void raft_fixture_close(struct raft_fixture *f)
 {
     unsigned i;
@@ -1324,7 +1333,9 @@ static void fireTick(struct raft_fixture *f, unsigned i)
     f->event.server_index = i;
     f->event.type = RAFT_FIXTURE_TICK;
     io->next_tick += io->tick_interval;
-    io->tick_cb(io->io);
+    if (f->servers[i].alive) {
+        io->tick_cb(io->io);
+    }
 }
 
 /* Complete the first request with completion time @t on the @i'th server. */
@@ -1761,6 +1772,22 @@ static void disconnectFromAll(struct raft_fixture *f, unsigned i)
     }
 }
 
+static void reconnectToAll(struct raft_fixture *f, unsigned i)
+{
+    unsigned j;
+    for (j = 0; j < f->n; j++) {
+        if (j == i) {
+            continue;
+        }
+        /* Don't reconnect to disconnected peers */
+        if (!f->servers[j].alive) {
+            continue;
+        }
+        raft_fixture_desaturate(f, i, j);
+        raft_fixture_desaturate(f, j, i);
+    }
+}
+
 bool raft_fixture_saturated(struct raft_fixture *f, unsigned i, unsigned j)
 {
     struct raft_io *io1 = &f->servers[i].io;
@@ -1781,6 +1808,12 @@ void raft_fixture_kill(struct raft_fixture *f, unsigned i)
     f->servers[i].alive = false;
 }
 
+void raft_fixture_revive(struct raft_fixture *f, unsigned i)
+{
+    reconnectToAll(f, i);
+    f->servers[i].alive = true;
+}
+
 int raft_fixture_grow(struct raft_fixture *f, struct raft_fsm *fsm)
 {
     unsigned i;
