@@ -7,7 +7,7 @@
 
 #include "../../include/raft.h"
 
-void FsmInit(struct raft_fsm *fsm);
+void FsmInit(struct raft_fsm *fsm, int version);
 
 void FsmClose(struct raft_fsm *fsm);
 
